import pytest

from pypayment import Payment


def test_default_url():
    with pytest.raises(TypeError):
        payment = Payment(1)
        payment.url


def test_default_status():
    with pytest.raises(TypeError):
        payment = Payment(1)
        payment.status



import pytest

from pypayment import PayOkPayment, Payment, PaymentStatus, AuthorizationError, NotAuthorized
from test import payok_api_key, payok_api_id, payok_shop_id, payok_shop_secret_key


def test_payment_creation_without_authorization():
    with pytest.raises(NotAuthorized):
        PayOkPayment(1)


def test_authorization_with_invalid_api_key():
    with pytest.raises(AuthorizationError):
        PayOkPayment.authorize("invalid data", payok_api_id, payok_shop_id, payok_shop_secret_key)


def test_authorization_with_invalid_api_id():
    with pytest.raises(AuthorizationError):
        PayOkPayment.authorize(payok_api_key, 0, payok_shop_id, payok_shop_secret_key)


def test_authorization_with_invalid_shop_id():
    with pytest.raises(AuthorizationError):
        PayOkPayment.authorize(payok_api_key, payok_api_id, 0, payok_shop_secret_key)


def test_authorization_with_invalid_shop_secret_key():
    with pytest.raises(AuthorizationError):
        PayOkPayment.authorize(payok_api_key, payok_api_id, payok_shop_id, "invalid data")


def test_authorization():
    PayOkPayment.authorize(payok_api_key, payok_api_id, payok_shop_id, payok_shop_secret_key)
    assert PayOkPayment.authorized


def test_payment_creation():
    payment: Payment = PayOkPayment(1)
    assert payment is not None


def test_payment_creation_with_float():
    payment: Payment = PayOkPayment(1.23)
    assert payment is not None


def test_url_getting():
    payment: Payment = PayOkPayment(1)
    assert "https://payok.io/pay?" in payment.url


def test_status_getting():
    payment: Payment = PayOkPayment(1.23)
    assert payment.status == PaymentStatus.WAITING



from enum import Enum


class PaymentStatus(Enum):
    WAITING = 0
    """Payment has been created, but has not yet been paid."""
    PAID = 1
    """Payment was successfully paid."""
    REJECTED = 2
    """Payment was rejected."""
    EXPIRED = 3
    """Payment has expired."""



import hashlib
import urllib.parse
from enum import Enum
from typing import Optional

import requests

from pypayment import Payment, PaymentStatus, NotAuthorized, PaymentGettingError, AuthorizationError


class PayOkPaymentType(Enum):
    CARD = "cd"
    """Payment with bank card."""
    QIWI = "qw"
    """Payment with QIWI."""
    YOOMONEY = "ym"
    """Payment with YooMoney."""
    WEBMONEY = "wm"
    """Payment with WebMoney."""
    PAYEER = "pr"
    """Payment with Payeer."""
    PERFECT_MONEY = "pm"
    """Payment with Perfect Money."""
    ADVCASH = "ad"
    """Payment with Advcash."""
    BEELINE = "bl"
    """Payment with Beeline."""
    MEGAFON = "mg"
    """Payment with Megafon."""
    TELE2 = "tl"
    """Payment with Tele2."""
    MTS = "mt"
    """Payment with MTS."""
    QIWI_MOBILE = "qm"
    """Payment with QIWI Mobile."""
    BITCOIN = "bt"
    """Payment with Bitcoin."""
    LITECOIN = "lt"
    """Payment with Litecoin."""
    DOGECOIN = "dg"
    """Payment with Dogecoin."""
    DASH = "ds"
    """Payment with Dash."""
    ZCASH = "zc"
    """Payment with ZСash."""


class PayOkCurrency(Enum):
    RUB = "RUB"
    """Russian ruble."""
    UAH = "UAH"
    """Ukrainian hryvnia."""
    USD = "USD"
    """United States dollar."""
    EUR = "EUR"
    """Euro."""
    RUB2 = "RUB2"
    """Russian ruble. (Alternative Gateway)"""


class PayOkPayment(Payment):
    authorized = False
    _api_key: str
    _api_id: int
    _shop_id: int
    _shop_secret_key: str
    _payment_type: PayOkPaymentType
    _currency: PayOkCurrency
    _success_url: Optional[str] = None
    _BASE_URL = "https://payok.io"
    _PAY_URL = _BASE_URL + "/pay"
    _API_URL = _BASE_URL + "/api"
    _TRANSACTION_URL = _API_URL + "/transaction"

    def __init__(self,
                 amount: float,
                 description: str = "",
                 payment_type: Optional[PayOkPaymentType] = None,
                 currency: Optional[PayOkCurrency] = None,
                 success_url: Optional[str] = None):
        """
        You need to PayOkPayment.authorize() first!

        Instantiation generates new PayOk invoice instance right away.

        Passed parameters will be applied to instance, but won't override default ones.

        :param amount: The amount to be invoiced.
        :param payment_type: PayOkPaymentType enum.
        :param currency: PayOkCurrency enum.
        :param success_url: User will be redirected to this url after paying.

        :raise NotAuthorized: When class was not authorized with PayOkPayment.authorize()
        :raise PaymentCreationError: When payment creation failed.
        """
        if not PayOkPayment.authorized:
            raise NotAuthorized("You need to authorize first: PayOkPayment.authorize()")

        self._payment_type = PayOkPayment._payment_type if payment_type is None else payment_type
        self._currency = PayOkPayment._currency if currency is None else currency
        self._success_url = PayOkPayment._success_url if success_url is None else success_url

        super().__init__(amount, description)
        self.id: str = str(self.id[:16])
        if not description:
            self.description = self.id

        self._url = self._create()

    @classmethod
    def authorize(cls,
                  api_key: str,
                  api_id: int,
                  shop_id: int,
                  shop_secret_key: str,
                  payment_type: PayOkPaymentType = PayOkPaymentType.CARD,
                  currency: PayOkCurrency = PayOkCurrency.RUB,
                  success_url: Optional[str] = None) -> None:
        """
        Must be called before the first use of the class!

        Tries to authorize to PayOk API.
        Saves passed parameters as default.

        :param api_key: API key from https://payok.io/cabinet/api.php
        :param api_id: ID of API key from https://payok.io/cabinet/api.php
        :param shop_id: ID of shop from https://payok.io/cabinet/main.php
        :param shop_secret_key: Secret key of shop from https://payok.io/cabinet/main.php
        :param payment_type: PayOkPaymentType enum.
        :param currency: PayOkCurrency enum.
        :param success_url: User will be redirected to this url after paying.

        :raise PaymentCreationError: When authorization fails.
        """
        PayOkPayment._api_key = api_key
        PayOkPayment._api_id = api_id
        PayOkPayment._shop_id = shop_id
        PayOkPayment._shop_secret_key = shop_secret_key
        PayOkPayment._payment_type = payment_type
        PayOkPayment._currency = currency
        PayOkPayment._success_url = success_url

        cls._try_authorize()

    @classmethod
    def _try_authorize(cls) -> None:
        data = {
            "API_ID": cls._api_id,
            "API_KEY": cls._api_key,
            "shop": cls._shop_id,
        }
        try:
            response = requests.post(cls._TRANSACTION_URL, data=data)
        except Exception as e:
            raise AuthorizationError(e)

        if response.json().get("status") == "error":
            raise AuthorizationError(response.json())

        data = {
            "amount": 1,
            "payment": "test",
            "shop": PayOkPayment._shop_id,
            "desc": "test",
            "currency": "RUB",
        }
        sign_str = "|".join(map(str, (data["amount"], data["payment"], data["shop"], data["currency"], data["desc"], PayOkPayment._shop_secret_key)))
        data["sign"] = hashlib.md5(sign_str.encode()).hexdigest()
        try:
            response = requests.post(PayOkPayment._PAY_URL, data=data)
        except Exception as e:
            raise AuthorizationError(e)

        if "Неверная подпись." in response.text:
            raise AuthorizationError("Invalid shop secret key")

        cls.authorized = True

    def _create(self) -> str:
        data = {
            "amount": self.amount,
            "payment": self.id,
            "shop": PayOkPayment._shop_id,
            "desc": self.description,
            "currency": self._currency.value,
            "success_url": self._success_url,
            "method": self._payment_type.value
        }

        sign_str = "|".join(map(str, (data["amount"], data["payment"], data["shop"], data["currency"], data["desc"], PayOkPayment._shop_secret_key)))
        data["sign"] = hashlib.md5(sign_str.encode()).hexdigest()

        return PayOkPayment._PAY_URL + "?" + urllib.parse.urlencode(data)

    @property
    def url(self) -> str:
        return self._url

    @property
    def status(self) -> PaymentStatus:
        data = {
            "API_ID": PayOkPayment._api_id,
            "API_KEY": PayOkPayment._api_key,
            "shop": PayOkPayment._shop_id,
            "payment": self.id
        }
        try:
            response = requests.post(PayOkPayment._TRANSACTION_URL, data=data).json()
        except Exception as e:
            raise PaymentGettingError(e)

        if response.get("status") == "error" and response.get("error_code") == "10":
            return PaymentStatus.WAITING

        if response.get("status") == "success":
            if response.get("1").get("transaction_status") == "1":
                return PaymentStatus.PAID
            else:
                return PaymentStatus.WAITING

        raise PaymentGettingError(response)



class NotAuthorized(Exception):
    """Raised when payment provider class has not been authorized."""


class AuthorizationError(Exception):
    """Raised when authorization failed."""


class PaymentCreationError(Exception):
    """Raised when payment creation failed."""


class PaymentGettingError(Exception):
    """Raised when payment getting failed."""



from .exceptions import NotAuthorized, PaymentCreationError, PaymentGettingError, AuthorizationError
from .status import PaymentStatus
from .commission import ChargeCommission
from .payment import Payment
from .qiwi import QiwiPayment, QiwiPaymentType
from .yoomoney import YooMoneyPayment, YooMoneyPaymentType
from .payok import PayOkPayment, PayOkPaymentType, PayOkCurrency



from abc import ABC, abstractmethod
from uuid import uuid4

from pypayment import PaymentStatus


class Payment(ABC):
    """Payment interface than allows to create and check invoices."""
    def __init__(self, amount: float, description: str = ""):
        self.amount = amount
        """The amount to be invoiced."""
        self.description = description
        """Payment comment."""
        self.id = str(uuid4())
        """Unique Payment ID (generated with uuid4)."""

    @property
    @abstractmethod
    def url(self) -> str:
        """
        :return: Link to the created payment form.
        """
        pass

    @property
    @abstractmethod
    def status(self) -> PaymentStatus:
        """
        Requests the payment status from the payment provider

        :return: Payment status
        """
        pass
